@@ -63,6 +63,7 @@ public:
     UString& operator= (UString const &a_cstr) ;
     bool operator! () const ;
     vector<UString> split (const UString &a_delim) const ;
+    vector<UString> split_set (const UString &a_delim_set) const ;
     static UString join (const vector<UString> &a_elements,
                          const UString &a_delim=" ") ;
     static UString join (vector<UString>::const_iterator &a_from,
@@ -72,6 +73,8 @@ public:
 
     UString::size_type get_number_of_lines () const ;
 
+    UString::size_type get_number_of_words () const ;
+
     UString& printf (const UString &a_format, ...) ;
 
     UString& vprintf (const UString &a_format, va_list a_args);
