@@ -36,7 +36,6 @@
 #include "common/nmv-dynamic-module.h"
 #include "common/nmv-safe-ptr-utils.h"
 #include "nmv-i-conf-mgr.h"
-#include "nmv-i-lang-trait.h"
 
 NEMIVER_BEGIN_NAMESPACE (nemiver)
 
@@ -53,6 +52,7 @@ using std::string;
 using std::map;
 using std::list;
 
+class ILangTrait;
 class IDebugger;
 typedef SafePtr<IDebugger, ObjectRef, ObjectUnref> IDebuggerSafePtr;
 
@@ -78,18 +78,28 @@ public:
 
     /// \brief a breakpoint descriptor
     class BreakPoint {
+    public:
+
+        enum Type {
+            UNDEFINED_TYPE = 0,
+            STANDARD_BREAKPOINT_TYPE,
+            WATCHPOINT_TYPE
+        };
+
+    private:
         int m_number;
         bool m_enabled;
         UString m_address;
         UString m_function;
+        UString m_expression;
         UString m_file_name;
         UString m_file_full_name;
-        int m_line;
         UString m_condition;
+        Type m_type;
+        int m_line;
         int m_nb_times_hit;
 
     public:
-
         BreakPoint () {clear ();}
 
         /// \name accessors
@@ -107,6 +117,9 @@ public:
         const UString& function () const {return m_function;}
         void function (const UString &a_in) {m_function = a_in;}
 
+        const UString& expression () const {return m_expression;}
+        void expression (const UString &a_expr) {m_expression = a_expr;}
+
         const UString& file_name () const {return m_file_name;}
         void file_name (const UString &a_in) {m_file_name = a_in;}
 
@@ -131,6 +144,10 @@ public:
             }
             return false;
         }
+
+        Type type () const {return m_type;}
+        void type (Type a_type) {m_type = a_type;}
+
         /// @}
 
         /// \brief clear this instance of breakpoint
@@ -229,7 +246,10 @@ public:
         UString m_library;
     public:
 
-        Frame () {clear ();}
+        Frame () :
+            m_level (0),
+            m_line (0)
+        {}
 
         /// \name accessors
 
@@ -277,8 +297,20 @@ public:
 
     class Variable;
     typedef SafePtr<Variable, ObjectRef, ObjectUnref> VariableSafePtr;
+    typedef list<VariableSafePtr> VariableList;
     class Variable : public Object {
-        list<VariableSafePtr> m_members;
+        // non copyable.
+        Variable (const Variable &);
+        Variable& operator= (const Variable &);
+
+        bool operator == (Variable &a_other) const;
+        bool operator == (Variable &a_other);
+
+        VariableList m_members;
+        // If this variable was created with a backend counterpart
+        // (e.g: backend side variable objects in GDB), then this
+        // is the name of the backend side counterpart of this variable.
+        UString m_internal_name;
         UString m_name;
         UString m_name_caption;
         UString m_value;
@@ -288,8 +320,27 @@ public:
         //it can be dereferenced. The variable
         //it points to is stored in m_dereferenced
         VariableSafePtr m_dereferenced;
+        unsigned int m_num_expected_children;
+        // The expression with which this variable
+        // Can be referenced in the debugger.
+        // If empty, it can be set by calling
+        // IDebugger::query_variable_path_expr()
+        UString m_path_expression;
 
     public:
+        Variable (const UString &a_internal_name,
+                  const UString &a_name,
+                  const UString &a_value,
+                  const UString &a_type) :
+            m_internal_name (a_internal_name),
+            m_name (a_name),
+            m_value (a_value),
+            m_type (a_type),
+            m_parent (0),
+            m_num_expected_children (0)
+
+        {
+        }
 
         Variable (const UString &a_name,
                   const UString &a_value,
@@ -297,21 +348,92 @@ public:
             m_name (a_name),
             m_value (a_value),
             m_type (a_type),
-            m_parent (0)
+            m_parent (0),
+            m_num_expected_children (0)
 
         {
         }
 
         Variable (const UString &a_name) :
             m_name (a_name),
-            m_parent (0)
+            m_parent (0),
+            m_num_expected_children (0)
         {}
 
         Variable () :
-            m_parent (0)
+            m_parent (0),
+            m_num_expected_children (0)
         {}
 
-        const list<VariableSafePtr>& members () const {return m_members;}
+        const VariableList& members () const {return m_members;}
+
+        /// Returns the Nth member variable, if any.
+        /// Please note that this function has O(n) complexity. So use it with
+        /// care.
+        /// \param a_index, the index of the member variable to return,
+        ///        starting from zero.
+        /// \param a_var the output parameter. Is set if and only if the
+        ///         the function returned true.
+        /// \return true if there was a variable at the specified index,
+        ///  false otherwise.
+        bool get_member_at (int a_index,
+                            VariableSafePtr &a_var) const
+        {
+            int i = 0;
+            for (VariableList::const_iterator it = members ().begin ();
+                 it != m_members.end ();
+                 ++it, ++i) {
+                if (i == a_index) {
+                    a_var = *it;
+                    return true;
+                }
+            }
+            return false;
+        }
+
+        /// \return the sibbling index of the variable node.
+        /// If first sibbling at a given level has index 0.
+        int sibling_index () const
+        {
+            if (!parent ())
+                return 0;
+            VariableList::const_iterator it;
+            int i = 0;
+            for (it = parent ()->members ().begin ();
+                 it != parent ()->members ().end ();
+                 ++it, ++i) {
+                if (*it == this)
+                    return i;
+            }
+            THROW ("fatal: should not be reached");
+        }
+
+        /// Tests value equality between two variables.
+        /// Two variables are considered equal by value if their
+        /// respective memebers have the same values and same type.
+        /// \param a_other the other variable to test against
+        /// \return true if a_other equals the current instance
+        ///         by value.
+        bool equals_by_value (const Variable &a_other) const
+        {
+            if (name () != a_other.name ()
+                || type () != a_other.type ())
+                return false;
+            if (members ().empty () != a_other.members ().empty ())
+                return false;
+            VariableList::const_iterator it0, it1;
+            for (it0 = members ().begin (), it1 = a_other.members ().begin ();
+                 it0 != members ().end ();
+                 ++it0, ++it1) {
+                if (it1 == a_other.members ().end ())
+                    return false;
+                if (!(*it0)->equals_by_value (**it1))
+                    return false;
+            }
+            if (it1 != a_other.members ().end ())
+                return false;
+            return true;
+        }
 
         void append (const VariableSafePtr &a_var)
         {
@@ -320,6 +442,19 @@ public:
             a_var->parent (this);
         }
 
+        /// If this variable was created with a backend counterpart
+        /// (e.g. backend side variable objects in GDB), then this
+        /// returns the name of the backend side counterpart of this variable.
+        /// \return the name of the backend side counterpart of this
+        /// variable. It can be an empty string if there is no backend side
+        /// counterpart variable object created for this object.
+        const UString& internal_name () const {return m_internal_name;}
+
+        /// Set the name of the backend side counterpart object.
+        /// (e.g. variable object in GDB).
+        /// \param a_in the new name of backend side counterpart variable object.
+        void internal_name (const UString &a_in) {m_internal_name = a_in;}
+
         const UString& name () const {return m_name;}
         void name (const UString &a_name)
         {
@@ -337,12 +472,40 @@ public:
         void type (const UString &a_type) {m_type = a_type;}
         void type (const string &a_type) {m_type = a_type;}
 
-        Variable* parent () const {return m_parent;}
+        /// Return true if this instance of Variable has a parent variable,
+        /// false otherwise.
+        bool has_parent () const
+        {
+            return m_parent != 0;
+        }
+
+        /// A getter of the parent Variable of the current instance.
+        const VariableSafePtr parent () const
+        {
+            VariableSafePtr parent (m_parent, true/*add a reference*/);
+            return parent;
+        }
+
+        /// Parent variable setter.
+        /// We don't increase the ref count of the
+        /// parent variable to avoid memory leaks due to refcounting cyles.
+        /// The parent already holds a reference on this instance of
+        /// Variable so we really should not hold a reference on it as
+        /// well.
         void parent (Variable *a_parent)
         {
             m_parent = a_parent;
         }
 
+        /// \return the root parent of this variable.
+        const VariableSafePtr root () const
+        {
+            if (!has_parent ()) {
+                return VariableSafePtr (this, true /*increase refcount*/);
+            }
+            return parent ()->root ();
+        }
+
         void to_string (UString &a_str,
                         bool a_show_var_name = false,
                         const UString &a_indent_str="") const
@@ -351,6 +514,10 @@ public:
                 if (name () != "") {
                     a_str += a_indent_str + name ();
                 }
+
+                if (!internal_name ().empty ()) {
+                    a_str += "(" + internal_name () + ")";
+                }
             }
             if (value () != "") {
                 if (a_show_var_name) {
@@ -363,7 +530,7 @@ public:
             }
             UString indent_str = a_indent_str + "  ";
             a_str += "\n" + a_indent_str + "{";
-            list<VariableSafePtr>::const_iterator it;
+            VariableList::const_iterator it;
             for (it = members ().begin (); it != members ().end (); ++it) {
                 if (!(*it)) {continue;}
                 a_str += "\n";
@@ -376,7 +543,7 @@ public:
         void build_qname (UString &a_qname) const
         {
             UString qname;
-            if (parent () == 0) {
+            if (!parent ()) {
                 a_qname = name ();
                 if (!a_qname.raw ().empty () && a_qname.raw ()[0] == '*') {
                     a_qname.erase (0, 1);
@@ -395,6 +562,21 @@ public:
             }
         }
 
+        void build_qualified_internal_name (UString &a_qname) const
+        {
+            UString qname;
+            if (!parent ()) {
+                a_qname = internal_name ();
+            } else if (parent ()) {
+                parent ()->build_qname (qname);
+                qname.chomp ();
+                qname += "." + name ();
+                a_qname = qname;
+            } else {
+                THROW ("should not be reached");
+            }
+        }
+
         void set_dereferenced (VariableSafePtr a_derefed)
         {
             m_dereferenced  = a_derefed;
@@ -422,7 +604,7 @@ public:
                 return false;
             }
 
-            list<VariableSafePtr>::const_iterator it1, it2;
+            VariableList::const_iterator it1, it2;
             //first make sure our members have the same types as their members
             for (it1=members ().begin (), it2=a_other.members ().begin ();
                  it1 != members ().end ();
@@ -442,8 +624,8 @@ public:
             m_name = a_other.m_name;
             m_name_caption = a_other.m_name_caption;
             m_value = a_other.m_value;
-            list<VariableSafePtr>::iterator it1;
-            list<VariableSafePtr>::const_iterator it2;
+            VariableList::iterator it1;
+            VariableList::const_iterator it2;
             for (it1=m_members.begin (), it2=a_other.m_members.begin ();
                  it1 != m_members.end ();
                  it1++, it2++) {
@@ -457,7 +639,7 @@ public:
             m_name = a_other.m_name;
             m_value = a_other.m_value;
             m_type = a_other.m_type;
-            list<VariableSafePtr>::const_iterator it;
+            VariableList::const_iterator it;
             m_members.clear ();
             for (it = a_other.m_members.begin ();
                  it != a_other.m_members.end ();
@@ -468,6 +650,62 @@ public:
                 append (var);
             }
         }
+
+        unsigned int num_expected_children () const
+        {
+            return m_num_expected_children;
+        }
+
+        void num_expected_children (unsigned int a_in)
+        {
+            m_num_expected_children = a_in;
+        }
+
+        bool has_expected_children () const
+        {
+            return m_num_expected_children != 0;
+        }
+
+        /// \return true if the current variable needs to be unfolded
+        /// by a call to IDebugger::unfold_variable()
+        bool needs_unfolding () const
+        {
+            return (has_expected_children () && members ().empty ());
+        }
+
+        /// Return the descendant of the current instance of Variable.
+        /// \param a_internal_path the internal fully qualified path of the
+        ///        descendant variable.
+        const VariableSafePtr get_descendant
+                                (const UString &a_internal_path) const
+        {
+            VariableSafePtr result;
+            if (internal_name () == a_internal_path) {
+                result.reset (this, true /*take refcount*/);
+                return result;
+            }
+            for (VariableList::const_iterator it = m_members.begin ();
+                 it != m_members.end ();
+                 ++it) {
+                if (*it && (*it)->internal_name () == a_internal_path) {
+                    return *it;
+                }
+                result = (*it)->get_descendant (a_internal_path);
+                if (result) {
+                    return result;
+                }
+            }
+            return result;
+        }
+
+        const UString& path_expression () const
+        {
+            return m_path_expression;
+        }
+        void path_expression (const UString &a_expr)
+        {
+            m_path_expression = a_expr;
+        }
     };//end class Variable
 
     enum State {
@@ -601,38 +839,43 @@ public:
                                             current_frame_signal () const = 0;
 
 
-    virtual sigc::signal<void, const list<VariableSafePtr>&, const UString& >&
+    virtual sigc::signal<void, const VariableList&, const UString& >&
                             local_variables_listed_signal () const = 0;
 
-    virtual sigc::signal<void, const list<VariableSafePtr>&, const UString& >&
+    virtual sigc::signal<void, const VariableList&, const UString& >&
                             global_variables_listed_signal () const = 0;
 
+    /// Emitted as the result of the IDebugger::print_variable_value() call.
     virtual sigc::signal<void,
                          const UString&/*variable name*/,
-                         const VariableSafePtr&/*variable*/,
+                         const VariableSafePtr /*variable*/,
                          const UString& /*cookie*/>&
                                              variable_value_signal () const = 0;
+
+    /// Emitted as the result of the IDebugger::get_variable_value() call.
     virtual sigc::signal<void,
-                         const VariableSafePtr&/*variable*/,
+                         const VariableSafePtr /*variable*/,
                          const UString& /*cookie*/>&
                                      variable_value_set_signal () const = 0;
 
     virtual sigc::signal<void,
                          const UString&/*variable name*/,
-                         const VariableSafePtr&/*variable*/,
+                         const VariableSafePtr /*variable*/,
                          const UString& /*cookie*/>&
                                     pointed_variable_value_signal () const = 0;
 
+    /// Emitted as the result of the IDebugger::print_variable_type() call.
     virtual sigc::signal<void,
                          const UString&/*variable name*/,
                          const UString&/*type*/,
                          const UString&/*cookie*/>&
                                         variable_type_signal () const = 0;
     virtual sigc::signal<void,
-                         const VariableSafePtr&/*variable*/,
+                         const VariableSafePtr /*variable*/,
                          const UString&/*cookie*/>&
                                     variable_type_set_signal () const=0;
 
+    /// Emitted as a the result of the IDebugger::dereference_variable is call.
     virtual sigc::signal<void,
                          const VariableSafePtr/*the variable we derefed*/,
                          const UString&/*cookie*/>
@@ -681,6 +924,24 @@ public:
                           const std::vector<uint8_t>&,/*values*/
                           const UString& >&
                                           set_memory_signal () const = 0;
+
+    virtual sigc::signal<void, const VariableSafePtr, const UString&>&
+                                        variable_created_signal () const = 0;
+
+    virtual sigc::signal<void, const VariableSafePtr, const UString&>&
+                                        variable_deleted_signal () const = 0;
+
+    virtual sigc::signal<void, const VariableSafePtr, const UString&>&
+                                        variable_unfolded_signal () const = 0;
+
+    virtual sigc::signal<void, const VariableSafePtr, const UString&>&
+                variable_expression_evaluated_signal () const = 0;
+
+    virtual sigc::signal<void, const VariableList&, const UString&>&
+                changed_variables_signal () const  = 0;
+
+    virtual sigc::signal<void, VariableSafePtr, const UString&>&
+                assigned_variable_signal () const = 0;
     /// @}
 
     virtual void do_init (IConfMgrSafePtr &a_conf_mgr) = 0;
@@ -730,7 +991,9 @@ public:
 
     virtual void get_target_info (const UString &a_cookie="") = 0;
 
-    virtual ILangTraitSafePtr get_language_trait () = 0;
+    virtual ILangTrait& get_language_trait () = 0;
+
+    virtual bool is_variable_editable (const VariableSafePtr a_var) const = 0;
 
     virtual void do_continue (const UString &a_cookie="") = 0;
 
@@ -738,6 +1001,8 @@ public:
 
     virtual IDebugger::State get_state () const = 0;
 
+    virtual int get_current_frame_level () const = 0;
+
     virtual bool stop_target () = 0;
 
     virtual void exit_engine () = 0;
@@ -760,6 +1025,12 @@ public:
     virtual void set_breakpoint (const UString &a_func_name,
                                  const UString &a_condition="",
                                  const UString &a_cookie="") = 0;
+
+    virtual void set_watchpoint (const UString &a_expression,
+                                 bool a_write = true,
+                                 bool a_read = false,
+                                 const UString &a_cookie = "") = 0;
+
     virtual void set_catch (const UString &a_event,
                             const UString &a_cookie="") = 0;
 
@@ -794,7 +1065,9 @@ public:
     virtual void select_frame (int a_frame_id,
                                const UString &a_cookie="") = 0;
 
-    virtual void list_frames (const UString &a_cookie="") = 0;
+    virtual void list_frames (int a_low_frame=-1,
+                              int a_high_frame=-1,
+                              const UString &a_cookie="") = 0;
 
     virtual void list_frames_arguments (int a_low_frame=-1,
                                         int a_high_frame=-1,
@@ -845,6 +1118,60 @@ public:
             const std::vector<uint8_t>& a_bytes,
             const UString& a_cookie="") = 0;
 
+    typedef sigc::slot<void, const VariableSafePtr> ConstVariableSlot;
+    typedef sigc::slot<void, const VariableList> ConstVariableListSlot;
+    typedef sigc::slot<void, const UString&> ConstUStringSlot;
+
+    virtual void create_variable (const UString &a_name,
+                                  const UString &a_cookie = "") = 0;
+
+    virtual void create_variable (const UString &a_name,
+                                  const ConstVariableSlot &a_slot,
+                                  const UString &a_cookie = "") = 0;
+
+    virtual void delete_variable (const VariableSafePtr a_var,
+                                  const UString &a_cookie = "") = 0;
+
+    virtual void delete_variable (const VariableSafePtr a_var,
+                                  const ConstVariableSlot&,
+                                  const UString &a_cookie = "") = 0;
+
+    virtual void unfold_variable (VariableSafePtr a_var,
+                                  const UString &a_cookie = "") = 0;
+    virtual void unfold_variable
+                (VariableSafePtr a_var,
+                 const ConstVariableSlot&,
+                 const UString &a_cookie = "") = 0;
+
+    virtual void assign_variable (const VariableSafePtr a_var,
+                                  const UString &a_expression,
+                                  const UString &a_cookie = "") = 0;
+
+    virtual void assign_variable
+                    (const VariableSafePtr a_var,
+                     const UString &a_expression,
+                     const ConstVariableSlot &a_slot,
+                     const UString &a_cookie="") = 0;
+
+    virtual void evaluate_variable_expr (const VariableSafePtr a_var,
+                                         const UString &a_cookie = "") = 0;
+    virtual void evaluate_variable_expr
+            (const VariableSafePtr a_var,
+             const ConstVariableSlot &a_slot,
+             const UString &a_cookie = "")= 0;
+
+    virtual void list_changed_variables (VariableSafePtr a_root,
+                                         const UString &a_cookie = "") = 0;
+    virtual void list_changed_variables
+            (VariableSafePtr a_root,
+             const ConstVariableListSlot &a_slot,
+             const UString &a_cookie="") = 0;
+
+    virtual void query_variable_path_expr (const VariableSafePtr a_var,
+                                           const UString &a_cookie = "") = 0;
+    virtual void query_variable_path_expr (const VariableSafePtr a_var,
+                                           const ConstVariableSlot &a_slot,
+                                           const UString &a_cookie = "") = 0;
 
 };//end IDebugger
 
