@@ -110,7 +110,7 @@ struct CallStack::Priv {
     sigc::connection on_selection_changed_connection;
     Gtk::Widget *callstack_menu;
     Glib::RefPtr<Gtk::ActionGroup> call_stack_action_group;
-    int cur_frame_index;
+    unsigned cur_frame_index;
     unsigned nb_frames_expansion_chunk;
     int frame_low;
     int frame_high;
@@ -199,27 +199,11 @@ struct CallStack::Priv {
         return is_visible;
     }
 
-    Gtk::Widget* load_menu (UString a_filename, UString a_widget_name)
-    {
-        NEMIVER_TRY
-        string relative_path = Glib::build_filename ("menus", a_filename);
-        string absolute_path;
-        THROW_IF_FAIL (perspective.build_absolute_resource_path
-                (Glib::locale_to_utf8 (relative_path),
-                 absolute_path));
-
-        workbench.get_ui_manager ()->add_ui_from_file
-            (Glib::locale_to_utf8 (absolute_path));
-
-        NEMIVER_CATCH
-        return workbench.get_ui_manager ()->get_widget (a_widget_name);
-    }
-
     Gtk::Widget* get_call_stack_menu ()
     {
         if (!callstack_menu) {
-            callstack_menu = load_menu ("callstackpopup.xml",
-                                        "/CallStackPopup");
+            callstack_menu = perspective.load_menu ("callstackpopup.xml",
+                                                    "/CallStackPopup");
             THROW_IF_FAIL (callstack_menu);
         }
         return callstack_menu;
@@ -245,6 +229,7 @@ struct CallStack::Priv {
         }
 
         cur_frame_index = (*a_row_iter)[columns ().frame_index];
+        THROW_IF_FAIL (cur_frame_index < frames.size ());
         cur_frame = frames[cur_frame_index];
         THROW_IF_FAIL (cur_frame.level () >= 0);
         in_set_cur_frame_trans = true;
@@ -451,10 +436,10 @@ struct CallStack::Priv {
         Gtk::Menu *menu = dynamic_cast<Gtk::Menu*> (get_call_stack_menu ());
         THROW_IF_FAIL (menu);
 
-        //only pop up a menu if a row exists at that position
+        // only pop up a menu if a row exists at that position
         Gtk::TreeModel::Path path;
-        Gtk::TreeViewColumn* column=NULL;
-        int cell_x=0, cell_y=0;
+        Gtk::TreeViewColumn* column = 0;
+        int cell_x = 0, cell_y = 0;
         if (widget->get_path_at_pos (static_cast<int> (a_event->x),
                                      static_cast<int> (a_event->y),
                                      path,
@@ -572,6 +557,8 @@ struct CallStack::Priv {
     {
         LOG_FUNCTION_SCOPE_NORMAL_DD;
 
+        THROW_IF_FAIL (!a_frames.empty ());
+
         int dest_start_index = a_frames[0].level (),
             dest_end_index = a_frames.size () + dest_start_index - 1;
         unsigned level = 0;
